
//////////////////////////////////////////////////
//												//
//		MusicVillage Definition header file	//
//			2004 (C)DreamROM					//
//							Last Update:04/02/28//
//												//
//////////////////////////////////////////////////

#ifndef _MVILLAGE_HEADER_
#define _MVILLAGE_HEADER_

#ifndef _DLL_HDR_
#define __DLLEXPORT						__stdcall
#define __DLLPRIVATE					__cdecl

#define DLLVERSION_MAJOR(v)			(DWORD)(((DWORD) (v) & 0xFF000000) >> 24)
#define DLLVERSION_MINOR(v)			(DWORD)(((DWORD) (v) & 0x00FF0000) >> 16)
#define DLLVERSION_BUILD(v)			(DWORD)(((DWORD) (v) & 0x0000FFFF) >> 0)

#endif	//  _DLL_HDR_

//  Version of Musicvillage
//  You had better check these version using MVGetVersion function at the top of program.
#define MV_MAJOR_VERSION			0x0002
#define MV_MINOR_VERSION			0x0006
#define MV_BUILD_NUMBER				0x0023

//  Type definition
typedef LONG MVRESULT;
typedef BOOL (CALLBACK * LPFNMVCALLBACK)(DWORD dwMsg,DWORD dwParam,DWORD dwUser);

DECLARE_HANDLE(HMVSOUND);

//  MusicVillage constant values
#define MV_SYSEX_GM					0L		//  Reset by GM system
#define MV_SYSEX_GS					-1L		//  Reset by GS system
#define MV_SYSEX_XG					-2L		//  Reset by XG system
#define MV_NO_SINGLE				-1L		//  Not use single instrument
#define MV_MAX_STRING				0x80
#define MV_INVALID_DEVICE_ID		-2L
#define MV_MIN_TEMPO				0.25f
#define MV_MAX_TEMPO				2.00f
#define MV_MIN_INSTRUMENT			0L
#define MV_MAX_INSTRUMENT			127L
#define MV_MIN_FADE_TIME			1000
#define MV_MAX_FADE_TIME			10000
#define MV_DEFAULT_TIMEOUT			1000UL
#define MV_MIN_TIMEOUT				100UL
#define MV_MAX_TIMEOUT				1000UL
#define MV_MIN_VOLUME				0UL
#define MV_MAX_VOLUME				100UL

//  Constant error values - there will be returned by the function calling.
#define MVERR_NOERROR				0L
#define MVERR_SUCCESS				MVERR_NOERROR
#define MVERR_RETURNOK				MVERR_NOERROR
#define MVERR_LACKMEMORY			10L
#define MVERR_SYSTEMERROR			11L
#define MVERR_INITIALIZE			12L
#define MVERR_RETURNCANCEL			13L
#define MVERR_LOAD_STRING			14L
#define MVERR_FAILED_SET_TIMER		19L
#define MVERR_FILE_OPEN				20L
#define MVERR_FILE_INVALID_FORMAT	21L
#define MVERR_FILE_NOT_PCM			22L
#define MVERR_UNSUPPORTED_FORMAT	29L
#define MVERR_DEVICE_OPENED			30L
#define MVERR_DEVICE_CLOSED			31L
#define MVERR_DEVICE_NO_ENTRY		32L
#define MVERR_DEVICE_INVALID_NUMBER	33L
#define MVERR_DEVICE_CANNOT_OPEN	34L
#define MVERR_WAVE_DEVICE_NO_ENTRY	35L
#define MVERR_NOW_PLAYING			40L
#define MVERR_NOT_PLAYING			41L
#define MVERR_TIMEOUT				42L
#define MVERR_INVALID_COMMAND		43L
#define MVERR_INVALID_STATE			44L
#define MVERR_INVALID_HANDLE		45L
#define MVERR_INVALID_PARAMETER		46L
#define MVERR_INVALID_CAPS			47L
#define MVERR_NOBUFFERONMEMORY		48L
#define MVERR_INVALID_CALL_INTERVAL	49L
#define MVERR_NOW_FADINGOUT			50L
#define MVERR_NOW_FADINGIN			51L
#define MVERR_FILE_TOO_LARGE		52L

//  Types of MusicVillage capabilities
typedef enum {
	MVCAPS_ACTIONFLAG				= 0x00,		//  For more information, see MVAF_*
	MVCAPS_CALLBACKPTR				= 0x01,
	MVCAPS_USERPARAM				= 0x02,
	MVCAPS_TEMPO					= 0x03,
	MVCAPS_FADEOUT					= 0x04,		//	Set fade out, and Get command returns true or false of being fade out.
	MVCAPS_FADEIN					= 0x05,		//  Set fade in, and Get command returns true or false of being fade in.
	MVCAPS_SINGLE					= 0x06,
	MVCAPS_TIMEOUT					= 0x07,
	MVCAPS_SHORTMSG					= 0x08,		//  Send a short message to device through this function, and get a last message sent.
	MVCAPS_VOLUME					= 0x09,		//  Set and get muster volume
}MVCAPSTYPE;

//  Callback messages / sent in dwMsg params	//  Params
#define MVMSG_DEVICE_OPENERROR		0xFF		//  (LONG)DeviceNumber
#define MVMSG_DEVICE_OPEN			0x00		//  (LONG)DeviceNumber
#define MVMSG_DEVICE_CLOSE			0x01		//  NULL
#define MVMSG_LOAD_FILE				0x02		//  (LPCSTR)lpFileName
#define MVMSG_PLAY					0x03		//  NULL
#define MVMSG_STOP					0x04		//  NULL
#define MVMSG_WAIT					0x05		//  NULL
#define MVMSG_RESTART				0x06		//  (DWORD)Restart Point
#define MVMSG_LOOP					0x07		//  NULL
#define MVMSG_FADEOUT				0x08		//  NULL
#define MVMSG_FADEIN				0x09		//  NULL

//  Constant values of state in MusicVillage
#define MVS_EMPTY					-1		//  Device has not opened.
#define MVS_NOMSG					0		//  Device has opened, but music data has not read.
#define MVS_PLAYING					1
#define MVS_WAIT					2
#define MVS_STOP					3

//  Flags of MVCAPS_ACTIONFLAG
#define MVAF_GMRESET				0x0001	//  --------
#define MVAF_GSRESET				0x0002	//  Cannot exclusive.
#define MVAF_XGRESET				0x0004	//  --------
#define MVAF_NOSENDSYSEX			0x0010

//  Flags for CDDLG struct in dwFlags member
//    NOTE: Under 7 bits are masks to access to the member
#define CDDF_HWNDPARENT				0x0001
#define CDDF_DEVICENAME				0x0002
#define CDDF_HOOKPROC				0x0004
//    NOTE: Upper 8 bits are flags of actions
#define CDDF_RESERVED				0x0100
#define CDDF_INITIALIZE_DEVICE		0x0200
#define CDDF_HIDEMIDIMAPPER			0x0400
#define CDDF_ACTIVEUPDATE			0x0800

//  Play back flags for Wave sound.
#define PBO_PLAYBACK				0x80000000		//  This flag is used in MusicVillage. A Client application don't have to use this.
#define PBO_USE_LOOP_BEGIN			0x00000001		//  Enable the chunk 'LPBn'
#define PBO_USE_LOOP_END			0x00000002		//  Enable the chunk 'LBEd'
#define PBO_LOOP					0x00000004		//  Sound is repeated when the playing finishes.

//  Structs
typedef struct MVStatusInfo{
	DWORD dwSize;
	LONG  nUsedDevice;
	LONG  nState;
	DWORD dwCurPos;
	DWORD dwMusicLength;
	FLOAT fTempo;
	LPFNMVCALLBACK lpfnFunc;
	DWORD dwUser;
}MVSTATUSINFO, * LPMVSTATUSINFO;

typedef struct ChooseDeviceDlg{
	DWORD dwSize;
	DWORD dwFlags;
	HWND hWndParent;
	LONG nDevice;
	LPSTR lpszDeviceName;
	DLGPROC lpfnHookProc;
}CDDLG, * LPCDDLG;


//  Exported function
void __DLLEXPORT MusicVillageInitialize();
MVRESULT __DLLEXPORT MVOpenDevice(LONG nDev,HANDLE hEvent);
MVRESULT __DLLEXPORT MVCloseDevice();
MVRESULT __DLLEXPORT MVLoadMusic(LPSTR lpFileName);
MVRESULT __DLLEXPORT MVPlayMusic();
MVRESULT __DLLEXPORT MVStopMusic();
MVRESULT __DLLEXPORT MVWaitMusic();
MVRESULT __DLLEXPORT MVRestartMusic();
MVRESULT __DLLEXPORT MVGetStatus(LPMVSTATUSINFO lpsi);
MVRESULT __DLLEXPORT MVSendSysEx(LONG nLength,CHAR *lpData);

MVRESULT __DLLEXPORT MVSeek(DWORD dwPos);
MVRESULT __DLLEXPORT MVSetCaps(MVCAPSTYPE MVCaps,DWORD dwParam);
MVRESULT __DLLEXPORT MVGetCaps(MVCAPSTYPE MVCaps,DWORD *lpParam);
MVRESULT __DLLEXPORT MVGetVersion(DWORD *lpVersion);
MVRESULT __DLLEXPORT MVGetIcon(HICON *lpIcon);
MVRESULT __DLLEXPORT MVGetErrorString(DWORD dwError,LPSTR lpReturnString,DWORD dwMaxLength);
MVRESULT __DLLEXPORT MVChooseDeviceDialog(LPCDDLG lpcdd);
MVRESULT __DLLEXPORT MVGetBufferPtr(DWORD dwFlags,DWORD **ppBuffer,DWORD *lpLength);

MVRESULT __DLLEXPORT MVOpenSound(HMVSOUND *lpMVHandle,LONG nDev,LPSTR lpFileName);
MVRESULT __DLLEXPORT MVCloseSound(HMVSOUND hMVSound);
MVRESULT __DLLEXPORT MVPlaySound(HMVSOUND hMVSound,DWORD dwOption);
MVRESULT __DLLEXPORT MVStopSound(HMVSOUND hMVSound);
MVRESULT __DLLEXPORT MVWaitSound(HMVSOUND hMVSound);
MVRESULT __DLLEXPORT MVRestartSound(HMVSOUND hMVSound);

#endif	//  _MVILLAGE_HEADER_


